import java.util.*;
import java.util.concurrent.Callable;

/**
 * {@code Grid} instances represent the grid in <i>The Game of Life</i>.
 */
public class Grid implements Iterable<Cell> {

    private final int numberOfRows;
    private final int numberOfColumns;
    private final Cell[][] cells;

    /**
     * Creates a new {@code Grid} instance given the number of rows and columns.
     *
     * @param numberOfRows    the number of rows
     * @param numberOfColumns the number of columns
     * @throws IllegalArgumentException if {@code numberOfRows} or {@code numberOfColumns} are
     *                                  less than or equal to 0
     */
    public Grid(int numberOfRows, int numberOfColumns) {
        this.numberOfRows = numberOfRows;
        this.numberOfColumns = numberOfColumns;
        this.cells = createCells();
    }

    @Override
    public Iterator<Cell> iterator() {
        return new GridIterator(this);
    }

    private Cell[][] createCells() {
        Cell[][] cells = new Cell[getNumberOfRows()][getNumberOfColumns()];
        for (int rowIndex = 0; rowIndex < getNumberOfRows(); rowIndex++) {
            for (int columnIndex = 0; columnIndex < getNumberOfColumns(); columnIndex++) {
                cells[rowIndex][columnIndex] = new Cell();
            }
        }
        return cells;
    }

    /**
     * Returns the {@link Cell} at the given index.
     *
     * <p>Note that the index is wrapped around so that a {@link Cell} is always returned.
     *
     * @param rowIndex    the row index of the {@link Cell}
     * @param columnIndex the column index of the {@link Cell}
     * @return the {@link Cell} at the given row and column index
     */
    public Cell getCell(int rowIndex, int columnIndex) {
        return cells[getWrappedRowIndex(rowIndex)][getWrappedColumnIndex(columnIndex)];
    }

    private int getWrappedRowIndex(int rowIndex) {
        return (rowIndex + getNumberOfRows()) % getNumberOfRows();
    }

    private int getWrappedColumnIndex(int columnIndex) {
        return (columnIndex + getNumberOfColumns()) % getNumberOfColumns();
    }

    /**
     * Returns the number of rows in this {@code Grid}.
     *
     * @return the number of rows in this {@code Grid}
     */
    public int getNumberOfRows() {
        return numberOfRows;
    }

    /**
     * Returns the number of columns in this {@code Grid}.
     *
     * @return the number of columns in this {@code Grid}
     */
    public int getNumberOfColumns() {
        return numberOfColumns;
    }

    /**
     * Transitions all {@link Cell}s in this {@code Grid} to the next generation.
     *
     * <p>The following rules are applied:
     * <ul>
     * <li>Any live {@link Cell} with fewer than two live neighbours dies, i.e. underpopulation.</li>
     * <li>Any live {@link Cell} with two or three live neighbours lives on to the next
     * generation.</li>
     * <li>Any live {@link Cell} with more than three live neighbours dies, i.e. overpopulation.</li>
     * <li>Any dead {@link Cell} with exactly three live neighbours becomes a live cell, i.e.
     * reproduction.</li>
     * </ul>
     */
    void nextGeneration() {
        goToNextState(calculateNextStates());
    }





    private boolean[][] calculateNextStates() {
        boolean [][] MyMatrice=new boolean [numberOfRows][numberOfColumns];
        for(int i=0; i<numberOfRows;i++){
            for (int j=0; j<numberOfColumns;j++){
                MyMatrice[i][j]= calculateNextState(i,j);
            }
        }



           return MyMatrice; }


    private boolean calculateNextState(int rowIndex, int columnIndex) {
	    int alive=countAliveNeighbours(rowIndex, columnIndex);
	    Cell c=getCell(rowIndex,columnIndex);
	    if (c.isAlive()){
            if ((alive == 2) || (alive == 3)){
                return true;
            }
            else{
                return false;
            }
        }
        else {
            if(alive == 3){
                return true;
            }
            else{
                return false;
            }

        }

    }

    private int countAliveNeighbours(int rowIndex, int columnIndex) {
        List<Cell> neighbours = getNeighbours(rowIndex, columnIndex);
        int count=0;
        for (Cell i: neighbours) {
            if (i.isAlive()) {
                count++;
            }
        }
        return count;
    }








    private List<Cell> getNeighbours(int rowIndex, int columnIndex) {
        List<Cell> voisines = new ArrayList<>();
        Cell c = getCell(rowIndex,columnIndex);
        voisines.add(c);
        Cell v1=getCell(rowIndex-1,columnIndex-1);
        voisines.add(v1);
        Cell v2=getCell(rowIndex-1,columnIndex);
        voisines.add(v2);
        Cell v3=getCell(rowIndex-1,columnIndex+1);
        voisines.add(v3);
        Cell v4=getCell(rowIndex,columnIndex-1);
        voisines.add(v4);
        Cell v5=getCell(rowIndex,columnIndex+1);
        voisines.add(v5);
        Cell v6=getCell(rowIndex+1,columnIndex-1);
        voisines.add(v6);
        Cell v7=getCell(rowIndex+1,columnIndex);
        voisines.add(v7);
        Cell v8=getCell(rowIndex+1,columnIndex+1);
        voisines.add(v8);

        return voisines;
    }

    private void goToNextState(boolean[][] nextState) {
        nextState=calculateNextStates();
        for (int i=0;i<numberOfRows;i++){
            for(int j=0;j<numberOfColumns;j++){
                if(nextState[i][j]==true){
                    cells[i][j].setAlive();
                }
                else{
                    cells[i][j].setDead();
                }
            }
        }


    }

    /**
     * Sets all {@link Cell}s in this {@code Grid} as dead.
     */
    void clear() {
        for(int i=0;i<numberOfRows;i++){
            for(int j=0;j<numberOfColumns;j++){
                cells[i][j].setDead();
            }
        }
    }

    /**
     * Goes through each {@link Cell} in this {@code Grid} and randomly sets it as alive or dead.
     *
     * @param random {@link Random} instance used to decide if each {@link Cell} is alive or dead
     * @throws NullPointerException if {@code random} is {@code null}
     */
 
    void randomGeneration(Random random) {
       boolean b;
        for(int i=0;i<numberOfRows;i++){
            for(int j=0;j<numberOfColumns;j++){
                b= random.nextBoolean();
                if(b){
                    cells[i][j].setAlive();
                }
                else{
                    cells[i][j].setDead();
                }
            }
        }

    }


}
